;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  
;; A silly happy block that grows larger if hit from below
;;
;;
;; EXTRA BIT: If set, block will skip the middle size and go directly to the largest size
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Palette = $0A
;sprite palette to use, from 08-0F
!SP = $01
;SP1/2 = $00,  SP3/4 = $01


!ShrinkTime = $80
;how long to wait until the block shrinks again


!SFX = $09
;sound effect to play (try 09, 10 or 1E on port 1DF9)
;$01 for the normal hitting your head sound
!Port = $1DF9
;port to play it in
;list is at http://smwc.me/96604

!Tiles = $C6
;the location of the top left corner tile in Lunar Magic's 8x8 editor
;order has to always be the same


;don't change these defines. or do, but it won't help much, they're just random sprite tables
!Size = $160E         ;current size of block, from 0-2

!State = $1594        ;0 = don't bother checking for blocks; 1 = check for blocks and spawn

!Expansion = $1602    ;how far the block is expanded at the moment
                      
!DesiredExp = $1626   ;how far the current state expands the block
                      ;state 0 = 0
                      ;state 1 = 4
                      ;state 2 = 8
                      
!ShrinkTimer = $1540  ;how long to wait till the block shrinks again

!Steps = $187B        ;whether or not the block skips the middle step
                      ;affected by extra bit

!Side = $1570         ;collision check's results
                      ;00 = no contact
                      ;01 = top
                      ;02 = bottom
                      ;FF = left
                      ;FE = right

!YLo = $157C
!YHi = $1534          ;preserve height for the jump

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
LDA $7FAB10,x        ; check for extra bit
AND #$04             ; (bit 2 -> %100 -> $4)
BEQ NoExtraIsSet     ; if not set, skip the following
INC !Steps,x
NoExtraIsSet:


LDA $D8,x
STA !YLo,x
LDA $14D4,x
STA !YHi,x
RTL
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB                     ; \
PHK                     ;  | The same old
PLB                     ; /
JSR MainCode            ;  Jump to sprite code
PLB                     ; Yep
RTL                     ; Return1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  Return11:
RTS

  MainCode:
JSR SubGFX
JSR SubOffScreen_X0	;handle off screen situation

LDA $9D             ;if sprites are locked, Return1
BNE Return11

;solid routine below
;fill in sprite slot A
LDA !Expansion,x
ASL          ;double the expansion value
STA $0C


LDA $E4,x
SEC
SBC !Expansion,x
STA $04
LDA $14E0,x
SBC #$00
STA $0A

LDA $D8,x
SEC
SBC $0C
STA $05
LDA $14D4,x
SBC #$00
STA $0B

LDA #$10
CLC
ADC $0C
STA $07       ;height
STA $06       ;width (are the same)


JSR SolidContact


  Shrinking:
LDA !ShrinkTimer,x
BNE .NoShrink

LDA !ShrinkTime
STA !ShrinkTimer,x

LDA #$01            ;don't check for collision while checking
STA !State,x

LDA !Size,x
BEQ .NoShrink
DEC                 ;dec state if it's not 0
SEC
SBC !Steps,x
STA !Size,x
TAX                 ;index by state
LDA Desireds,x      ;load desired expansion by state
LDX $15E9
STA !DesiredExp,x   ;store to desiredexp


  .NoShrink
LDA !State,x
BNE FitHeight       ;no collision checking if we're currently expanding/shrinking

LDA !YLo,x
CMP $D8,x
BNE FitHeight


  HitCheck:
LDA !Side,x
CMP #$02            ;check for bottom collision as per the SolidContact routine
BNE FitHeight


LDA #$01            ;don't expand more while it already is doing so
STA !State,x

LDA !Size,x
CMP #$02
BEQ .DullSound
LDA #!SFX
STA !Port
BRA .GoodSound
  .DullSound
LDA #$01
STA $1DF9
  .GoodSound
LDA #$F8
STA $AA,x

LDA !ShrinkTime
STA !ShrinkTimer,x

LDA !Size,x
CMP #$02
BCS FitHeight
INC                 ;inc state if it's not 2 (or more)
CLC
ADC !Steps,x
STA !Size,x
TAX                 ;index by state
LDA Desireds,x      ;load desired expansion by state
LDX $15E9
STA !DesiredExp,x   ;store to desiredexp

  FitHeight:        ;fit the expansion
LDA !Expansion,x
CMP !DesiredExp,x
BEQ .NoFit
BCC .IncHeight
;decrease size
DEC !Expansion,x
BRA .NoFit2
  .IncHeight
INC !Expansion,x
BRA .NoFit2
  .NoFit
STZ !State,x        ;zero out state if we reached what we wanted
  .NoFit2


  Jump:
LDA $14D4,x
STA $0D
LDA $D8,x
STA $0C

LDA !YHi,x
XBA
LDA !YLo,x
REP #$20
CMP $0C
SEP #$20
BEQ .NoJump
BCC .BelowPoint

INC $AA,x

BRA .NoJump


  .BelowPoint     ;if sprite is below origin point, put it on origin
LDA !YLo,x
STA $D8,x
LDA !YHi,x        ;restore y position
STA $14D4,x

STZ $AA,x         ;0 speed

  .NoJump
JSL $018022     ; \ Apply speed to position 
JSL $01801A     ; /

RTS

Desireds:
db $00,$04,$08



;;;;;
; LEOD'S SOLID SPRITE ROUTINE
; ASSUMES THAT SPRITE SLOT A IS FILLED IN
; Which side Mario is touching the sprite from is stored to $1570,x
; Values are: 00 = no contact; 01 = top; 02 = bottom; FE = left; FF = right
;;;;;

SolidContact:
JSL $03B664           ;fill in mario clipping

STZ $1570,x           ;no contact ram
JSL $03B72B           ;check if the filled in objects intersect
BCS .Contact          ;go to NoContact if no contact
RTS
  .Contact


;check if mario should ride the sprite before the edges so he isnt
;stuck on top because hes "inside" the sprite technically
JSR Get_Solid_Vert
LDY $187A               ;load yoshi flag into y
LDA $0F
CMP .HeightReq,y
BCS .DecideDownOrSides  ;if difference is less than FF-E6, mario isnt quite high enough
BPL .DecideDownOrSides  ;if difference is positive, mario is -below-

INC $1570,x             ;tell sprite that mario is touching from above (01)
                        ;INC works because it's re-set to 00 every frame

LDA #$01
STA $1471               ;set "on sprite" flag

LDA #$E1                ;load E1
LDY $187A
BEQ .NoYoshi
LDA #$D1                ;if on yoshi, load D1 instead (one block higher)
  .NoYoshi
CLC
ADC $05                 ;add sprite y position, low byte
SEC
SBC #$01                ;substract one to align mario
STA $96                 ;store to mario y position, low byte
LDA $0B                 ;load sprite y position, high byte
ADC #$FF                ;decrease by 1 or keep same depending on overflow
STA $97                 ;store to mario y position

RTS


.HeightReq        ;height difference requirement with and without yoshi
db $D4,$C6,$C6

  .DecideDownOrSides
LDA $0A
XBA               ;load left edge of sprite
LDA $04
REP #$20
SEC
SBC #$000C        ;take into account right edge
CMP $D1           ;compare to mario
SEP #$20
BCS .HorzCheck

REP #$20
CLC
ADC #$0008        ;get back left edge of sprite
STA $0C
SEP #$20

LDA $0C
CLC
ADC $06           ;add width to get right edge
STA $0C
LDA $0D
ADC #$00
STA $0D
REP #$20
LDA $0C
CMP $D1           ;compare to mario
SEP #$20
BCC .HorzCheck

;mario is touching the sprite from below
LDA $77           ;if mario is on floor, dont signal sprite that hes touching
AND #$04
BEQ .DoSignal     ;but still keep it solid

LDA $7D           ;load mario y speed
BPL .NoContact    ;if he is moving downwards, return


  .DoSignal
LDA #$02
STA $1570,x       ;tell sprite that mario is touching from below (02)

  .NoSignal
LDA #$08          ;store #$10 to mario speed
STA $7D

RTS

  .HorzCheck
;check if mario -was- outside the sprite horizontally
JSR Get_Solid_Horz
LDA $0F                 ;get range difference between sprite and mario
BMI .MarioLeft
;mario is to right of sprite

SEC
SBC $06                 ;substract width for left edge
BPL .NoContact

;mario is touching right
LDA #$FE
STA $1570,x        ;tell sprite mario is touching right (FE)

LDA $04
SEC
SBC #$02
CLC
ADC $06
STA $0C

LDA $0A
ADC #$00
STA $0D

REP #$20
LDA $0C
STA $94
SEP #$20


LDA $7B
BPL .NoContact      ;unless mario is moving left, zero out speed
STZ $7B

RTS

  .MarioLeft
;mario is to left of sprite
CLC
ADC #$10          ;add mario width
BMI .NoContact

;mario is touching left
DEC $1570,x       ;tell sprite mario is touching left (FF)
                  ;dec works because it's re-set to 00 every frame

LDA $04
SEC
SBC #$0D
STA $0C

LDA $0A
SBC #$00
STA $0D

REP #$20
LDA $0C
STA $94
SEP #$20

LDA $7B
BMI .NoContact      ;unless mario is moving right, zero out speed
STZ $7B

  .NoContact
RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Graphics Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

YHalve:     ;table that determines what to do with tiles on the y axis, more complex than x by far
db $01,$01,$02,$02            ;fills (top lr, bottom lr)

db $00,$00 :  db $03,$03      ;edges bottom (lr) top (lr)
db $02,$02 :  db $01,$01      ;side edges left right top, left right bottom
db $03,$03,$00,$00            ;corners (top lr, bottom lr)


GrowTiles:
db !Tiles+$12,!Tiles+$12,!Tiles+$12,!Tiles+$12     ;fills (top lr, bottom lr)

db !Tiles+$13,!Tiles+$13 :  db !Tiles+$4,!Tiles+$4 ;edges bottom (lr) top (lr)
db !Tiles+$3,!Tiles+$14 :  db !Tiles+$3,!Tiles+$14 ;side edges left right top, left right bottom
db !Tiles,!Tiles+$1,!Tiles+$10,!Tiles+$11          ;corners (top lr, bottom lr)

!FaceTile = $C8

FaceFlip:
db $00,$40                    ;face (lr)

FaceX:
db $FB,$04

  SubGFX:
JSR GET_DRAW_INFO
  PHX
  

LDA !Expansion,x      ;save us from messing with x
STA $08
LSR
CLC
ADC $08
STA $07               ;temp store face offset (expansion + 1/2 expansion)

;;;;;;;;;;;;;;;;; face tiles
LDX #$01
  FaceLoop:
PHX

LDA $00
CLC
ADC FaceX,x
ADC #$04
STA $0300,y

LDA $01           ;add sprite position
CLC
ADC #$04          ;move down 8 pixels to align with grid and back up 4 to fit on the block
SBC $07           ;add expansion
STA $0301,y       ;store to y pos

LDA #!FaceTile
STA $0302,y

LDA.b #!Palette-$08<<1|$20|!SP     ;load props + palette
CLC
ADC FaceFlip,x
STA $0303,y


INY
INY
INY
INY

PLX
DEX
BPL FaceLoop




;;;;;;;;;;;;;;;;; block tiles
LDX #$0F
  GrowLoop:
PHX

;----x disposition
LDA #$04
CLC
ADC $08  ;add expansion to standard block size
STA $09           ;temp store

TXA
CMP #$08          ;halve all the earlier slots (they're the inner ones horizontally)
BCS .NoHalve

LDA $08
LSR               ;halve offset
STA $09


.NoHalve
TXA
AND #$01
BNE .RightTile

LDA $00         ;load sprite x pos
SEC
SBC $09         ;substract adjusted expansion address
CLC
ADC #$04
STA $0300,y     ;store to oam x pos
  BRA .YOff

  .RightTile
LDA $00         ;load sprite x pos
CLC
ADC $09         ;substract adjusted expansion address
CLC
ADC #$04
STA $0300,y     ;store to oam x pos


;----y disposition
  .YOff
LDA #$08
CLC
ADC $08  ;add expansion to standard block size
STA $09           ;temp store

LDA YHalve,x    ;check what to do with this tile on the y axis
BEQ .NoYOff     ;if 00, then the tile stays in place at all times
CMP #$01
BEQ .LowerHalf  ;if 01, then the tile is in the lower half of the middle
CMP #$02
BEQ .UpperHalf  ;if 02, then the tile is in the upper half of the middle

  .TopTile
LDA $09
CLC
ADC $08         ;no math done on expansion, just expand all the way little one
STA $09
BRA .YDone

  .LowerHalf
LDA $09
CLC
ADC $08
JSR Div3
STA $09
BRA .YDone

  .UpperHalf
LDA $09
CLC
ADC $08
JSR Div3
STA $06
CLC
ADC $06
STA $09
BRA .YDone



  .NoYOff
STZ $09         ;no expansion

  .YDone
LDA $01         ;load sprite y pos
CLC
ADC #$07             ;adjust block to be as high as normal blocks, then move down 8
SEC
SBC $09
STA $0301,y     ;store to oam y pos

LDA GrowTiles,x ;load tile
STA $0302,y     ;store to oam tile

LDA.b #!Palette-$08<<1|$20|!SP     ;load props + palette
STA $0303,y


INY
INY
INY
INY

PLX
DEX
BMI .BreakLoop
JMP GrowLoop

  .BreakLoop
  PLX

LDY #$00        ; #$00 = 8x8, #$02 = 16x16, #$FF = variable
LDA #$11        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam
RTS


  Div3:         ;divide by 3
STA $07
LSR
ADC #21
LSR
ADC $07
ROR
LSR
ADC $07
ROR
LSR
ADC $07
ROR
LSR
RTS





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
        ;org $03B75C
 
TABLE1:              db $0C,$1C
TABLE2:              db $01,$02
 
GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
        STZ $15A0,x             ; reset sprite offscreen flag, horizontal
        LDA $E4,x               ; \
        CMP $1A                 ;  | set horizontal offscreen if necessary
        LDA $14E0,x             ;  |
        SBC $1B                 ;  |
        BEQ ON_SCREEN_X         ;  |
        INC $15A0,x             ; /
 
ON_SCREEN_X:         LDA $14E0,x             ; \
        XBA      ;  |
        LDA $E4,x               ;  |
        REP #$20                ;  |
        SEC      ;  |
        SBC $1A                 ;  | mark sprite INVALID if far enough off screen
        CLC      ;  |
        ADC.w #$0040            ;  |
        CMP.w #$0180            ;  |
        SEP #$20                ;  |
        ROL A                   ;  |
        AND #$01                ;  |
        STA $15C4,x             ; /
        BNE INVALID             ;
       
        LDY #$00                ; \ set up LOOP:
        LDA $1662,x             ;  |
        AND #$20                ;  | if not smushed (1662 & 0x20), go through LOOP twice
        BEQ ON_SCREEN_LOOP      ;  | else, go through LOOP once
        INY      ; /
ON_SCREEN_LOOP:      LDA $D8,x               ; \
        CLC      ;  | set vertical offscreen if necessary
        ADC TABLE1,y            ;  |
        PHP      ;  |
        CMP $1C                 ;  | (vert screen boundry)
        ROL $00                 ;  |
        PLP      ;  |
        LDA $14D4,x             ;  |
        ADC #$00                ;  |
        LSR $00                 ;  |
        SBC $1D                 ;  |
        BEQ ON_SCREEN_Y         ;  |
        LDA $186C,x             ;  | (vert offscreen)
        ORA TABLE2,y            ;  |
        STA $186C,x             ;  |
ON_SCREEN_Y:         DEY         ;  |
        BPL ON_SCREEN_LOOP      ; /
 
        LDY $15EA,x             ; get offset to sprite OAM
        LDA $E4,x               ; \
        SEC      ;  |
        SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
        STA $00                 ; /
        LDA $D8,x               ; \
        SEC      ;  |
        SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
        STA $01                 ; /
        RTS      ; Return1
 
INVALID:             PLA         ; \ Return1 from *main gfx routine* subroutine...
        PLA      ;  |    ...(not just this subroutine)
        RTS      ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, Return1
                    BEQ Return1_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, Return1
                    AND #$04                ; |
                    BNE Return1_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL Return1_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
Return1_35:          RTS                     ; Return1
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, Return1
                    AND #$04                ; |
                    BNE Return1_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS Return1_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL Return1_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / Return1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Ride sprite routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  RideY:
LDA #$01
STA $1471               ;set "on sprite" flag
STZ $7D                 ;zero out mario's y speed

LDA #$E1                ;load E1
LDY $187A
BEQ NoYoshi
LDA #$D1                ;if on yoshi, load D1 instead (one block higher)
  NoYoshi:
CLC
ADC $06                 ;add sprite y position, low byte
STA $96                 ;store to mario y position, low byte
LDA $07                 ;load sprite y position, high byte
ADC #$FF                ;decrease by 1 or keep same depending on overflow
STA $97                 ;store to mario y position

RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; vertical mario/sprite position check
; changed for a specific solid sprite routine to use filled-in clip values
; Y = 1 if mario above sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                ;org $03B829 

Get_Solid_Vert:       
LDY #$00        ;init as 0
LDA $D3         ;mario y pos
SEC
SBC #$10
SBC $05         ;minus sprite y pos
STA $0F         ;store range difference in 0F
LDA $D4         ;mario y pos high
SBC $0B         ;minus sprite y pos high + overflow
BPL MarioLower  ;if the result is not positive (so Mario would be below)
INY             ;increase y
  MarioLower:
STY $0E         ;store in $0E
RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; horizontal mario/sprite position check
; changed for a specific solid sprite routine to use filled-in clip values
; Y = 1 if mario left of sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Get_Solid_Horz:
LDY #$00        ;init as 0
LDA $D1         ;mario x pos
SEC
SBC $04         ;minus sprite x pos
STA $0F         ;store range difference in 0F
LDA $D2         ;mario x pos high
SBC $0A         ;minus sprite x pos high + overflow
BPL MarioRight  ;if the result is not positive (so Mario would be to the right)
INY             ;increase y
  MarioRight:
STY $0E         ;store in $0E
RTS
